'''
    PyBB - BBCode Processor

    Copyright (c) 2003-2004 Gang Seong Lee <gslee@mail.kw.ac.kr>
    All rights reserved, see COPYING for details.
'''

import re
import htmlValidate
import linkProcessor
from i18n import _
import bbslib

#############################################################
class CodeTag:
    tagList = 'quote b i u color size code url list email img title'.split()
    def __init__(self, tag, start):
        fields = tag.split('=')
        self.tagName = fields[0]
        self.arg = '='.join(fields[1:])
        self.content = ''
        self.stringIndex = start

    def updateContent(self, s):
        self.content += s

    def getIndex(self):
        return self.stringIndex
    
    def updateIndex(self, pos):
        self.stringIndex = pos
        
    def __nonzero__(self):
        return self.tagName in self.tagList
    
    def __repr__(self):
        return '[%s]' % self.tagName
        return '[%s=%s] #%s (%s)' % (self.tagName, self.arg, self.stringIndex, self.content)

    def __str__(self):
        return self.content

    def do_quote(self, escapeWhiteSpace, stack): # needs escape white space
        if escapeWhiteSpace and not hasWhiteSpaceProcessingCode(stack):
            newline1, s = splitFirstNewLine(self.content)
            s, newline2 = splitLastNewLine(s)
            s = newline1+bbslib.escapeWhiteSpace(s)+newline2
        else:
            s = self.content
        html = '<table><tr><td class=quote>%s</td></tr></table>' % s
        html = linkProcessor.linkProcessor(html)
        return html

    def do_b(self, escapeWhiteSpace, stack): # needs escape white space
        if escapeWhiteSpace and not hasWhiteSpaceProcessingCode(stack):
            s = bbslib.escapeWhiteSpace(self.content) or self.content
        else:
            s = self.content
        html = '<b>%s</b>' % s
        html = linkProcessor.linkProcessor(html)
        return html

    def do_i(self, escapeWhiteSpace, stack): # needs escape white space
        if escapeWhiteSpace and not hasWhiteSpaceProcessingCode(stack):
            s = bbslib.escapeWhiteSpace(self.content) or self.content
        else:
            s = self.content
        html = '<i>%s</i>' % s
        html = linkProcessor.linkProcessor(html)
        return html

    def do_u(self, escapeWhiteSpace, stack): # needs escape white space
        if escapeWhiteSpace and not hasWhiteSpaceProcessingCode(stack):
            s = bbslib.escapeWhiteSpace(self.content) or self.content
        else:
            s = self.content
        html = '<u>%s</u>' % s
        html = linkProcessor.linkProcessor(html)
        return html

    def do_color(self, escapeWhiteSpace, stack): # needs escape white space
        if not self.arg:
            raise SyntaxError, '[color=..] requires argument'
        if escapeWhiteSpace and not hasWhiteSpaceProcessingCode(stack):
            s = bbslib.escapeWhiteSpace(self.content) or self.content
        else:
            s = self.content
        html = '<font color=%s>%s</font>' % (self.arg, s)
        html = linkProcessor.linkProcessor(html)
        return html

    def do_size(self, escapeWhiteSpace, stack): # needs escape white space
        if escapeWhiteSpace and not hasWhiteSpaceProcessingCode(stack):
            s = bbslib.escapeWhiteSpace(self.content) or self.content
        else:
            s = self.content
        html = '<font size=%s>%s</font>' % (self.arg, s)
        html = linkProcessor.linkProcessor(html)
        return html

    def do_code(self, escapeWhiteSpace, stack):    # needs escape white space
        content = self.content
        if not hasWhiteSpaceProcessingCode(stack):
            newline, s = splitFirstNewLine(content)
            content = newline + bbslib.escapeWhiteSpace(s)
        html = '<table width="100%%"><tr><td class="code">%s</td></tr></table>' % content
        html = '<table><tr><td class="code">%s</td></tr></table>' % content
        return html

    def do_list(self, escapeWhiteSpace, stack):
        L = self.content.split('[*]')
        L = filter(lambda s: s.strip(), L)
        L = map(lambda s: '<li>%s</li>' % s.rstrip(), L)
        self.content = '\n'.join(L)
        if self.arg:
            html = '<ol type="%(arg)s">\n%(content)s\n</ol>' % self.__dict__
        else:
            html = '<ul>\n%(content)s\n</ul>' % self.__dict__
        html = linkProcessor.linkProcessor(html)
        return html

    def do_url(self, escapeWhiteSpace, stack):
        if self.arg:
            html = '<a href=%(arg)s>%(content)s</a>' % self.__dict__
        else:
            html = '<a href=%(content)s>%(content)s</a>' % self.__dict__
        return html

    def do_email(self, escapeWhiteSpace, stack):
        html = '<a href="emailto:%(content)s">%(content)s</a>' % self.__dict__
        return html

    def do_img(self, escapeWhiteSpace, stack):
        html = '<img src="%(content)s">' % self.__dict__
        return html

    def do_title(self, escapeWhiteSpace, stack):
        html = '''
<TABLE WIDTH=100%% BGCOLOR="#669933" CELLPADDING="1" CELLSPACING="0" BORDER="0">
<TR><TD>
<TABLE WIDTH=100%% BGCOLOR="#EEEEEE" CELLPADDING="4" CELLSPACING="0" BORDER="0">
<TR><TD VALIGN="MIDDLE">
%(content)s
</TD></TR>
</TABLE>
</TD></TR>
</TABLE>
''' % self.__dict__
        return html

def hasCode(L, codeName):
    for tag in L:
        if tag.tagName == codeName:
            return True
    return False

def hasOneOfCodes(L, codeList):
    for codeName in codeList:
        if hasCode(L, codeName):
            return True
    return False

def hasWhiteSpaceProcessingCode(stack):
    return hasOneOfCodes(stack, ('quote', 'b', 'i', 'u', 'color', 'size', 'code'))

def splitFirstNewLine(s):
    if s[:2] == '\r\n':
        return s[:2], s[2:]
    if s[0] in ('\r', '\n'):
        return s[0], s[1:]
    return '', s

def splitLastNewLine(s):
    if s[-2:] == '\r\n':
        return s[:-2], s[-2:]
    if s[-1] in ('\r', '\n'):
        return s[:-1], s[-1]
    return s, ''

def bbCodeProcessor(text, escapeWhiteSpace=False):  # escapeWhiteSpace out side of BB code
    p = re.compile('\[(.*?)\]')
#    codep = re.compile('\[(/code)\]')
    L = [CodeTag('__start__', 0)]
    codeSection = 0
    pos = 0
    while 1:
        m = p.search(text, pos)
        if m:
            tag = m.group(1)
            epos = m.end()
            if tag.startswith('/') and (tag[1:] in CodeTag.tagList):
                if tag == '/code':
                    codeSection -= 1
                if codeSection:
                    pos = epos
                    continue
                matchTag = L.pop()
                if matchTag.tagName != tag[1:]:
                    raise SyntaxError, _("[tag] doesn't match")+(' %s' % L)
                s = text[matchTag.getIndex():epos - (len(tag)+2)]
                matchTag.updateContent(s)
                matchTag.updateIndex(epos)
                action = getattr(CodeTag, 'do_'+matchTag.tagName)
                L[-1].updateContent(action(matchTag, escapeWhiteSpace, L))
                L[-1].updateIndex(matchTag.getIndex())
            else:
                codeTag = CodeTag(tag, epos)
                if codeSection:
                    if codeTag.tagName == 'code':
                        codeSection += 1
                    pos = epos
                    continue
                if codeTag:
                    if codeTag.tagName == 'code':
                        codeSection += 1
                    s = text[L[-1].getIndex():epos - (len(tag)+2)]  # outside of bb code
                    if escapeWhiteSpace and not hasWhiteSpaceProcessingCode(L):
                        s = bbslib.escapeWhiteSpace(s)
                    L[-1].updateContent(linkProcessor.linkProcessor(s))
                    L[-1].updateIndex(epos)
                    L.append(codeTag)
            pos = epos
        else:
            break
    if len(L) > 1:
        raise SyntaxError, _("BB code doesn't match")+' '+(' '.join(map(repr, L[1:])))
    s = text[L[0].getIndex():len(text)]
    if escapeWhiteSpace:
        s = bbslib.escapeWhiteSpace(s)
    L[0].updateContent(linkProcessor.linkProcessor(s))
    return L[0].content
