'''
    PyBB - Database Independent BBS library module.

    Copyright (c) 2003-2004 Gang Seong Lee <gslee@mail.kw.ac.kr>
    All rights reserved, see COPYING for details.
'''

from __future__ import generators

import os
import shutil
import time
import re
import weakref
from StringIO import StringIO
from urllib import urlretrieve
from urllib import quote
import types, glob
import cgi
import zipfile

import dblib
import config
import users
import exceptionLib
from templateProcessor import templateProcessor, getTemplate
import pyConfigParser
import htmlValidate
import linkProcessor
import bbcode
import cache
import userGroup
import mail

# Thread type
UNDEFINED, DEFAULT, QNA, CLOSED, NOTICE = range(5)

class ZipCRCError(Exception): pass
class BadGalleryFile(Exception): pass

groups = None
globalNS = None
articleCache = cache.Cache(config.maxArticleCache)

def getGlobalNS():
    global globalNS
    if globalNS:
        getStatistic()
        return globalNS
    class NameSpace(dict):
        def __init__(self, **kw):
            dict.__init__(self, kw)
        def __getattr__(self, name):
            return self[name]
    globalNS = NameSpace(
        user=None,
        session=None
        )
    globalNS.update(globals()['__builtins__'])
    d = config.__dict__
    for key, value in d.items():
        globalNS[key] = value
    globalNS['templateProcessor'] = templateProcessor
    globalNS['getTemplate'] = getTemplate
    globalNS['getCookedDateFormat'] = getCookedDateFormat
    globalNS['groups'] = getGroups()
    globalNS['strCutter'] = strCutter
    globalNS['config'] = config
    def escape(s, flag=True):
        return cgi.escape(s, flag)
    globalNS['escape'] = escape
    getStatistic()
    return globalNS

def getStatistic():
    globalNS['numberOfUsers'] = users.numberOfUsers()
    globalNS['numberOfActiveUsers'] = users.numberOfUsers(config.memberLevel)
    groups = getGroups()
    sum = 0
    sumOfnTopics = 0
    sumOfnPosts = 0
    latestPost = '0000-00-00 00:00:00'
    latestPoster = 'nobody'
    for category in groups:
        sum += len(category)
        for forum in category:
            sumOfnTopics += forum.nTopics
            if latestPost < forum.lastDate:
                latestPost = forum.lastDate
                latestPoster = forum.lastPoster
    globalNS['numberOfCategories'] = len(groups)
    globalNS['numberOfForums'] = sum
    globalNS['numberOfThreads'] = sumOfnTopics
    globalNS['latestPost'] = latestPost
    globalNS['latestPoster'] = latestPoster

def getGroups():
    global groups
    if groups:
        return groups
    else:
        groups = Groups()
        groups.refresh()
        return groups

def escapeTextForDb(text):
    text = text.replace('\\', '\\\\')
    text = text.replace("'", "\\'")
    return text

def escapeWhiteSpace(text):
    sp = re.compile(' ( +)')
    def spacerepl(match):
        s = match.group()
        return ' '+'&nbsp;'*(len(s)-1)
    text = text.expandtabs()
    text = sp.sub(spacerepl, text)
    text = text.replace('\n', ' <br/>\n')
    return text

def escapeTagMarker(s):
    s = s.replace('<:', '&lt;:')
    return s.replace(':>', ':&gt;')

def escapeTextForm(body):
    body = body.replace('<br/>','\n')
    body = body.replace('&', '&amp;')
    body = escapeTagMarker(body)
    return body

class Page(object):
    def __init__(self, pagesize, startPage=1):
        self.nPages = 0
        self.pagesize = int(pagesize)
        self.startPage = startPage

    def setnPages(self, N=-1):
        if N == -1:
            N = len(self)
        self.nPages = (N + self.pagesize - 1) / self.pagesize
#        self.pageIndex = [no*self.pagesize for no in range(self.nPages)]
        self.pageIndex = [(no-(self.startPage-1))*self.pagesize for no in range(self.nPages)]

    def page(self, no=1):
        pagesize = self.pagesize
        try:
            k = self.pageIndex[no-1]
            for i in range(k, min(k+pagesize, len(self))):
                yield self[i]
        except:
            raise StopIteration

class Tree:
    def __nonzero__(self):
        return True

    def setParent(self, parent):
        if parent != None:
            self.parent = weakref.ref(parent)
        else:
            self.parent = None
    def getParent(self):
        if self.parent == None:
            return None
        return self.parent()
    def makeOneNameSpace(self):
        L = []
        node = self
        while node != None:
            L.append(node.__dict__)
            node = node.getParent()
        d = {}
        while L:
            d.update(L.pop())
        return d

    def toHTML(self, teulPath=None, locals={}):
        if teulPath == None:
            teulPath = self.standard_html
        template = getTemplate(os.path.join(config.skinPath, teulPath))
        globalNS = getGlobalNS()
        d = self.makeOneNameSpace()
        d['self'] = self
        d['templateProcessor'] = templateProcessor
        d.update(locals)
        return templateProcessor(template, globalNS, d)

class Groups(list, Tree):
    '''List of Category'''
    standard_html = 'groups.teul'
    def __init__(self):
        self.parent = None
        self.nTotalPosts = 0
        for category in config.getCategories():
            self.append(Category(self, category))

    def updateDbTables(self):
        config.dbTables = []
        for child in self:
            config.dbTables.append(child.toList())
        self.saveGroupDescFile()

    def saveGroupDescFile(self):
        config.saveGroupDescFile()

    def addCategory(self, categoryName):
        if self.getCategoryIndexByName(categoryName) > -1:
            return False
        for char in ['\\', '<', '>', '?', ':', '"', '|', '/', '*']:
            if char in categoryName:
                return False
        self.append(Category(self, config.CategoryConfig([categoryName])))
        self.updateDbTables()
        return self[-1]

    def deleteCategory(self, categoryName):
        index = self.getCategoryIndexByName(categoryName)
        if index < 0:
            return False
        if len(self[index]):
            return False
        del self[index]
        self.updateDbTables()
        os.remove(os.path.join(config.systemFolder, config.encode2Hex(categoryName)+'.ini'))
        return True

    def moveUpCategory(self, categoryName):
        gIndex = self.getCategoryIndexByName(categoryName)
        if gIndex <= 0:
            return False
        moveCategory = self.pop(gIndex)
        self.insert(gIndex - 1, moveCategory)
        self.updateDbTables()
        return True

    def moveDownCategory(self, categoryName):
        gIndex = self.getCategoryIndexByName(categoryName)
        if gIndex < 0 or gIndex == len(self)-1:
            return False
        moveCategory = self.pop(gIndex)
        self.insert(gIndex + 1, moveCategory)
        self.updateDbTables()
        return True

    def changeCategoryName(self, oldCategoryName, newCategoryName):
        for char in ['\\', '<', '>', '?', ':', '"', '|', '/', '*']:
            if char in newCategoryName:
                return False

        category = self.getCategoryByName(oldCategoryName)
        if not category:
            return False
        else:
            category.changeCategoryName(newCategoryName)
            self.updateDbTables()
            return True


    def getCategoryIndexByName(self, categoryName):
        c = 0
        for child in self:
            if child.categoryName.lower() == categoryName.lower():
                return c
            c += 1
        return -1

    def getCategoryByName(self, categoryName):
        for child in self:
            if child.categoryName == categoryName:
                return child
        return False

    def getForum(self, forumId):
        for category in self:
            forum = category.getForumById(forumId)
            if isinstance(forum, Forum):
                return forum
        return False

    getForumById = getForum

    def getForumByName(self, forumName):
        for category in self:
            forum = category.getForumByName(forumName)
            if isinstance(forum, Forum):
                return forum
        return False

    def getForumList(self):
        forumList = []
        for category in self:
            for forum in category:
                forumList.append((forum.forumId, forum.forumName))
        return forumList
        
    def getThread(self, forumId, threadId):
        return self.getForum(forumId).getThreadById(threadId)

    def getArticle(self, forumId, threadId, articleId):
        return self.getForum(forumId).getThreadById(threadId).readArticle(articleId)

    def refresh(self):
        for child in self:
            child.refresh()
        self.updateStatistic()

    def updateStatistic(self):
        self.nTotalPosts = reduce(lambda a,b:a+b, [child.nCategoryPost for child in self], 0)


class Category(list, Tree):
    '''List of Forums'''
    standard_html = 'category.teul'
    def __init__(self, parent, category):
        self.setParent(parent)
        self.categoryName = category.getName()
        self.quotedCategoryName = quote(quote(self.categoryName))
        self.fold = category.fold
        for forum in category.getForumConfigs():
            self.append(Forum(self, **forum))
        self.nCategoryPost = 0

    def getVisibleForumList(self):
        return [forum for forum in self if 'hidden' not in forum.status.split()]

    def changeCategoryName(self, newCategoryName):
        groups = self.getParent()
        catNames = [cat.categoryName.strip() for cat in groups]
        if newCategoryName.strip() in catNames:
            return None
        self.categoryName = newCategoryName
        self.quotedCategoryName = quote(quote(newCategoryName))
        self.getParent().updateDbTables()

    def addForum(self, **kw):  # new forum
        if 'magicNumber' in kw:
            forumId = config.tablePrefix + '_forum_' + str(int(kw['magicNumber']))
            del kw['magicNumber']
        else:
            forumId = config.tablePrefix + '_forum_' + str(int(time.time()))
        ret = self.addForumById(forumId, **kw)
        if ret:
            return forumId
        return ret

    def addForumById(self, forumId, **kw):
        registeredTableNames = dblib.getTableNames(dblib.getCursor())
        if forumId in registeredTableNames:
            return False

        for forum in self:
            if forumId == forum.forumId:
                return False

        self.append(Forum(self, forumId, **kw))
        self.getParent().updateDbTables()
        dblib.makeTables()
        return True

    def deleteForumById(self, forumId):
        k = 0
        for forum in self:
            if forum.forumId == forumId:
                break
            k += 1
        else:
            return False
        forumName = forum.forumName
        dblib.removeTable(forum.forumId)
        dblib.removeTable(forum.forumId+'Thread')
        self.pop(k)
        self.getParent().updateDbTables()
        # update forumEmailNotice information
        '''
        for user in users.getUserList(pagesize=users.numberOfUsers(), updateCache=False):
            L = user.forumEmailNotice.split()
            changed = False
            while forumName in L:
                L.remove(forumName)
                changed = True
            if changed:
                user.setForumEmailNoticeList(L)
                user.update()
        '''
        return True

    def editForum(self, forumId, forumName, **kw):
        forumName = forumName.strip()
        index = 0
        for forum in self:
            if forumId == forum.forumId:
                break
            index += 1
        else:
            return False
        oldForumName = self[index].forumName
        self[index] = Forum(self, forumId, forumName, **kw)
        self[index].refresh()
        if oldForumName != forumName:
            try:
                uidList = file(os.path.join(config.mailNoticeFolder, 'MailNotice_%s') % forumId, 'r').read().split()
            except IOError:
                uidList = []
            for uid in uidList:
                user = users.getUser(uid)
                L = user.forumEmailNotice.strip().split('\n')
                if oldForumName in L:
                    k = L.index(oldForumName)
                    L[k] = forumName
                user.setForumEmailNoticeList(L)
                user.update()
        self.getParent().updateDbTables()
        self[index].refresh()
        return True

    def moveUpForum(self, forumId):
        fIndex = 0
        groups = self.getParent()
        cIndex = groups.getCategoryIndexByName(self.categoryName)
        for forum in self:
            if forumId == forum.forumId:
                break
            fIndex += 1
        else:
            return False

        if fIndex == 0 and cIndex == 0:
            return False
        elif fIndex == 0 and cIndex > 0:
            moveForum = self.pop(fIndex)
            category = groups[cIndex - 1]
            moveForum.setParent(category)
            category.append(moveForum)
        else:
            moveForum = self.pop(fIndex)
            self.insert(fIndex - 1, moveForum)
        groups.updateDbTables()
        return True

    def moveDownForum(self, forumId):
        nForum = len(self) - 1
        fIndex = 0
        groups = self.getParent()
        nCategory = len(groups) - 1
        cIndex = groups.getCategoryIndexByName(self.categoryName)
        for forum in self:
            if forumId == forum.forumId:
                break
            fIndex += 1
        else:
            return False

        if fIndex == nForum and cIndex == nCategory:
            return False
        elif fIndex == nForum and cIndex < nCategory:
            moveForum = self.pop(fIndex)
            category = groups[cIndex + 1]
            moveForum.setParent(category)
            category.insert(0, moveForum)
        else:
            moveForum = self.pop(fIndex)
            self.insert(fIndex + 1, moveForum)
        groups.updateDbTables()
        return True

    def refresh(self):
        for child in self:
            child.refresh()
        self.updateStatistic()

    def updateStatistic(self):
        self.nCategoryPost = reduce(lambda a,b:a+b, [child.nForumPost for child in self], 0)
        parent = self.getParent()
        if parent:
            parent.updateStatistic()

    def getForumIdListByName(self, forumName):
        L = []
        for forum in self:
            if forumName == forum.forumName:
                L.append(forum.forumId)

        if not L:
            return False

        return L

    def getForumById(self, forumId):
        for forum in self:
            if forum.forumId == forumId:
                return forum
        return False

    def getForumByName(self, forumName):
        for forum in self:
            if forum.forumName == forumName:
                return forum
        return False


    def toList(self):
        L = config.CategoryConfig([self.categoryName], self.fold)
        for child in self:
            L.append(child.toDict())
        return L


    
# Forum
################################################################################

class Forum(list, Tree, Page):
    '''List of Threads'''
    def __init__(self, parent, forumId, forumName='', forumDescription ='',
                 status='unlocked', moderator='',
                 defaultType='Discussion', skinType='default',
                 view='all', read='all', post='member', reply='member',
                 edit='private', delete='private', sort='mdate', order='desc',
                 userGroupList=None, emailNoticeUserGroupList=None,
                 thumbnailSizeX=128, thumbnailSizeY=96, thumbnailColumns=5,
                 numberOfTitlesInAPage=10, numberOfArticlesInAPage=10,
                 **kw):
        self.setParent(parent)
        Page.__init__(self, numberOfTitlesInAPage)
        self.forumId = forumId
        self.forumName = forumName
        self.forumDesc = forumDescription
        self.status = status
        self.moderator = moderator.replace(',', ' ')
        self.defaultType = defaultType
        self.skinType = skinType
        self.numberOfTitlesInAPage = int(numberOfTitlesInAPage)
        self.numberOfArticlesInAPage = int(numberOfArticlesInAPage)
        self.thumbnailSizeX = int(thumbnailSizeX)
        self.thumbnailSizeY = int(thumbnailSizeY)
        self.thumbnailColumns = int(thumbnailColumns)

        self.view = view
        self.read = read
        self.post = post
        self.reply = reply
        self.edit = edit
        self.delete = delete
        self.sortBy = sort
        self.order = order
        self.curSortBy = sort
        self.curOrder = order

        if type(userGroupList) == type(''):
            self.userGroupList = userGroupList.split()
        else:
            self.userGroupList = userGroupList or []
            
        if type(emailNoticeUserGroupList) == type(''):
            self.emailNoticeUserGroupList = emailNoticeUserGroupList.split()
        else:
            self.emailNoticeUserGroupList = emailNoticeUserGroupList or []

        self.forumSkinPath = 'forum_%s_%s/forum.teul' % (defaultType, skinType)
        self.threadTitleSkinPath = 'forum_%s_%s/threadTitle.teul' % (defaultType, skinType)
        self.threadSkinPath = 'forum_%s_%s/thread.teul' % (defaultType, skinType)
        self.articleSkinPath = 'forum_%s_%s/article1.teul' % (defaultType, skinType)
        self.articleSkinPath2 = 'forum_%s_%s/article2.teul' % (defaultType, skinType)
        self.newThreadSkin = 'forum_%s_%s/newThread.teul' % (defaultType, skinType)
        self.editSkin1 = 'forum_%s_%s/edit1.teul' % (defaultType, skinType)
        self.editSkin2 = 'forum_%s_%s/edit2.teul' % (defaultType, skinType)
        self.replySkin = 'forum_%s_%s/reply.teul' % (defaultType, skinType)
        self.standard_html = self.forumSkinPath

        self.nTopics = 0
        self.nForumPost = 0
        cursor = dblib.getCursor()
        self.noticeThreadList = []
        self.lastDate = ''
        self.lastPoster = ''

        self.threadIdIndexDict = {}
        self.cache = cache.Cache(config.maxThreadCachePerForum)

    def isUserGroupMember(self, user):
        registeredUserGroupConfig = userGroup.getUserGroup()
        for groupName in self.userGroupList:
            if registeredUserGroupConfig.isMember(groupName, user):
                return True
        return False

    def getNoticeList(self):
        return self.noticeThreadList

    def updateNoticeList(self):
        cursor = dblib.getCursor()
        cursor.execute('SELECT threadId FROM %sThread WHERE type = %s ORDER BY mdate DESC' % (self.forumId, NOTICE))
        threadIdList = dblib.extractNames(cursor.fetchall())
        self.noticeThreadList = [Thread(self, self.forumId, threadId) for threadId in threadIdList]
        return self.noticeThreadList

    def refresh(self, sortBy='', order='', pagesize=None):
        if pagesize == None:
            pagesize = self.numberOfTitlesInAPage
        cursor = dblib.getCursor()
        cursor.execute('SELECT COUNT(*) FROM %sThread WHERE type = %s' % (self.forumId, NOTICE))
        self.nNotices = cursor.fetchone()[0]
        if self.nNotices != len(self.noticeThreadList):
            self.updateNoticeList()
        
        self.pagesize = int(pagesize)
        if sortBy:
            self.curSortBy = sortBy
        if order:
            self.curOrder = order
        
        for noticeThread in self.getNoticeList():
            noticeThread.curSortBy = self.curSortBy
            noticeThread.curOrder = self.curOrder
            
        self.threadIdIndexDict.clear()
        self.updateStatistic()
        self.setnPages()
        self.cache.clear()
        if os.path.exists(os.path.join(config.skinPath, self.threadSkinPath)):
            for child in self:
                child.standard_html = self.threadSkinPath
            for child in self.noticeThreadList:
                child.standard_html = self.threadSkinPath
            
    def updateStatistic(self):
        cursor = dblib.getCursor()
        cursor.execute('SELECT COUNT(*) FROM %sThread' % self.forumId)
        self.nTopics = cursor.fetchone()[0]
        cursor.execute('SELECT COUNT(*) FROM %s' % self.forumId)
        self.nForumPost = cursor.fetchone()[0]

        self.lastDate = ''
        self.lastPoster = ''
        cursor.execute('SELECT max(mdate) FROM %sThread' % (self.forumId))
        mdate = cursor.fetchone()[0]
        if mdate:
            if type(mdate) in [type(''), type(None)]:
                self.lastDate = str(mdate)
            else:
                self.lastDate = str(mdate)[:-3]
            cursor.execute('SELECT lastPoster FROM %sThread WHERE mdate="%s"' % (self.forumId, self.lastDate))
            self.lastPoster = cursor.fetchone()[0]
        parent = self.getParent()
        if parent:
            parent.updateStatistic()

    def __len__(self):
        return self.nTopics - self.nNotices

    def __getitem__(self, index):
        thread = self.cache.getItem(index)
        if thread:
            return thread
        cursor = dblib.getCursor()
        try:
            sql = 'SELECT threadId FROM %sThread WHERE type != %s ORDER BY %s %s LIMIT %s, %s' % (
                self.forumId, NOTICE, self.curSortBy, self.curOrder, index, self.numberOfTitlesInAPage)
            cursor.execute(sql)
            threadIdList = dblib.extractNames(cursor.fetchall())
        except:
            raise IndexError, index
        k = 0
        for threadId in threadIdList:
            self.threadIdIndexDict[threadId] = index + k
            thread = Thread(self, self.forumId, threadId)
            self.cache.putItem(index+k, thread)
            k += 1
        return self.cache.getItem(index)

    def __iter__(self):
        for k in range(len(self)):
            yield self[k]

    def getThreadById(self, threadId):
        for thread, time in self.cache.values():
            if thread.threadId == threadId:
                return thread
        for thread in self.noticeThreadList:
            if thread.threadId == threadId:
                return thread
        return Thread(self, self.forumId, threadId) # FIXME

    def getNextThread(self, threadId):
        try:
            pos = self.threadIdIndexDict.get(threadId, -1)
            if pos >= 0:
                thread = self[pos+1]
                return thread.threadId, thread
            pos = 0
            for thread in self:
                if thread.threadId == threadId:
                    if thread.type == NOTICE:
                        return False, False
                    while self[pos+1].type == NOTICE:
                        pos += 1
                    break
                pos += 1
            thread = self[pos+1]
            return thread.threadId, thread
        except (IndexError, AttributeError), errMsg:
            return False, False

    def getPreviousThread(self, threadId):
        try:
            pos = self.threadIdIndexDict.get(threadId, -1)
            if pos >= 0:
                thread = self[pos-1]
                return thread.threadId, thread

            pos = 0
            for thread in self:
                if thread.threadId == threadId:
                    if thread.type == NOTICE:
                        return False, False
                    while self[pos-1].type == NOTICE:
                        pos -= 1
                    break
                pos += 1
            thread = self[pos-1]
            return thread.threadId, thread
        except (IndexError, AttributeError), errMsg:
            return 0, False

    def toDict(self):
        return config.ForumConfig({'forumId':self.forumId, 'forumName':self.forumName, 'forumDescription':self.forumDesc,
                'defaultType':self.defaultType,
                'skinType':self.skinType,
                'status':self.status, 'moderator':self.moderator,
                'view':self.view, 'read':self.read, 'post':self.post, 'reply':self.reply,
                'edit':self.edit, 'delete':self.delete, 'sortBy':self.sortBy, 'order':self.order,
                'userGroupList':' '.join(self.userGroupList), 'emailNoticeUserGroupList':' '.join(self.emailNoticeUserGroupList), 
                'thumbnailSizeX':self.thumbnailSizeX, 'thumbnailSizeY':self.thumbnailSizeY, 'thumbnailColumns':self.thumbnailColumns, 
                'numberOfArticlesInAPage':self.numberOfArticlesInAPage,
                'numberOfTitlesInAPage':self.numberOfTitlesInAPage,
                
                })

    def copyThread(self, thread):
        thread.refresh()
        oldThreadId = thread.threadId
        thread.threadId = 'NULL'
        cursor = dblib.getCursor()
        cursor.execute("INSERT INTO %sThread VALUES %s" % (self.forumId, thread.toSQL()))
        newThreadId = cursor.insert_id()
        thread.threadId = newThreadId
        names = dblib.getFieldNames(self.forumId)
        articleIdList = []
        for article in thread:
            orgThreadId = article.threadId
            orgArticleId = article.id
            article.threadId = newThreadId
            article.id = 'NULL'
            if article.upload1:
                fileStr = open(article.upload1, 'rb').read()
                fileUpload(self.forumId, article, fileStr)
            cursor.execute("INSERT INTO %s (%s) VALUES %s" % (self.forumId, ', '.join(names), article.toSQL(self.forumId)))
            writer = users.getUser(article.uid)
            if writer:
                writer.incrPostCount()
            articleIdList.append(article.id)
            article.threadId = orgThreadId
            article.id = orgArticleId
        self.refresh()
        thread.threadId = oldThreadId
        return newThreadId, articleIdList

    def moveThread(self, thread):
        newThreadId, articleIdList = self.copyThread(thread)
        deleteThread(thread.forumId, thread.threadId)
        return newThreadId, articleIdList

    def mergeThread(self, sourceId, targetId):  # FIXME
        srcThread = self.getThreadById(sourceId)
        srcThread.refresh()
        cursor = dblib.getCursor()
        names = dblib.getFieldNames(self.forumId)
        for article in srcThread:
            article.id = 'NULL'
            if article.upload1:
                fileName = os.path.split(article.upload1)[1]
                fileStr = file(article.upload1, 'rb').read()
            else:
                fileName = ''
                fileStr = ''
            replyArticle(self.forumId, article, targetId, fileName=fileName, fileStr=fileStr)
        deleteThread(self.forumId, sourceId)

    def addUserIdToForumEmailNoticeListFile(self, uid):
        if not os.path.exists(config.mailNoticeFolder):
            os.makedirs(config.mailNoticeFolder)
        try:
            uidList = file(os.path.join(config.mailNoticeFolder, 'MailNotice_%s') % self.forumId, 'r').read().split()
        except IOError:
            uidList = []
        if uid not in uidList:
            file(os.path.join(config.mailNoticeFolder, 'MailNotice_%s') % self.forumId, 'a').write('\n%s'%uid)

    def deleteUserIdFromForumEmailNoticeListFile(self, uid):
        if not os.path.exists(config.mailNoticeFolder):
            os.makedirs(config.mailNoticeFolder)
        try:
            uidList = file(os.path.join(config.mailNoticeFolder, 'MailNotice_%s') % self.forumId, 'r').read().split()
        except IOError:
            uidList = []
        except AttributeError:
            return
        try:
            while 1:
                uidList.remove(uid)
        except ValueError:
            pass
        file(os.path.join(config.mailNoticeFolder, 'MailNotice_%s') % self.forumId, 'w').write('\n'.join(uidList))

def checkForumAdmission(forum, permission, user, writer=None, topicStarter=''):
    permLevel = {'all':int(config.anonymousLevel), 'waiting':int(config.waitingLevel),
             'member':int(config.memberLevel), 'admin':int(config.adminLevel)}
    moderators = forum.moderator.split()
    if user.level >= int(config.adminLevel):
        return True
    if permission == 'moderator':
        if user.uid in moderators:
            return True
        else:
            return False
    if 'hidden' in forum.status.split():
        return False
    if permission == 'private':
        if (writer == user.uid) or (user.uid in moderators):
            return True
        else:
            return False
    if permission == 'topicStarter':
        if (topicStarter == user.uid) or (user.uid in moderators):
            return True
        else:
            return False
    if permission == 'userGroup':
        return forum.isUserGroupMember(user)
    level = permLevel[permission]
    if user.level >= level or user.uid in moderators:
        return True
    return False

class Message(object):
    def setCurrentDateAndTime(self):
        self.date = time.strftime(config.databaseDateFormat)

    def setModifiedDateAndTime(self):
        self.mdate = time.strftime(config.databaseDateFormat)

# Thread
################################################################################
class Thread(list, Message, Tree, Page):
    '''List of Article objects'''
    standard_html = 'thread.teul'
    def __init__(self, parent, forumId, threadId):
        self.setParent(parent)
        if parent:
            Page.__init__(self, self.getParent().numberOfArticlesInAPage)
        else:
            Page.__init__(self, 10)  # only for testcode
        cursor = dblib.getCursor()
        cursor.execute('SELECT * FROM %sThread WHERE threadId=%s' % (forumId, threadId))
        if cursor.rowcount < 1:
            raise exceptionLib.RecordNotFound, 'Thread RecordNotFound in DB forumId=%s threadId=%s' % (forumId, threadId)
        threadId, type, mdate, readCount, starter, lastPoster, replies, subject = cursor.fetchone()
        self.forumId = forumId
        self.threadId = threadId
        self.type = type
        self.mdate = str(mdate)
        self.starter = starter
        self.lastPoster = lastPoster
        self.readCount = readCount
        self.replies = replies
        self.subject = subject

    def refresh(self):
        if self.getParent():
            self.pagesize = int(self.getParent().numberOfArticlesInAPage)
        else:
            self.pagesize = 10    # only for testcode
        cursor = dblib.getCursor()
        cursor.execute('SELECT id FROM %s WHERE threadId=%s' % (self.forumId, self.threadId))
        articleIdList = list(cursor.fetchall())
        articleIdList.sort()
        self[:] = []
        for T in articleIdList:
            articleId = T[0]
            self.append(self.readArticle(articleId))
        self.setnPages()

    def incrReadCount(self):
        cursor = dblib.getCursor()
        cursor.execute("UPDATE %sThread SET readCount=readCount+1 WHERE threadId=%s" % (self.forumId, self.threadId))
        self.readCount += 1

    def decrReadCount(self):
        cursor = dblib.getCursor()
        cursor.execute("UPDATE %sThread SET readCount=readCount-1 WHERE threadId=%s" % (self.forumId, self.threadId))
        self.readCount -= 1

    def getArticleById(self, articleId):
        for child in self:
            if child.id == articleId:
                return child
        return False

    def getUserListToBeNotified(self):
        d = {}
        for article in self:
            if article.notify:
                d[article.uid] = None
        return d.keys()        
    
    def readArticle(self, id):
        cursor = dblib.getCursor()
        cursor.execute('SELECT * FROM %s WHERE id=%s' % (self.forumId, id))
        args = cursor.fetchone()
        fieldNames = dblib.getFieldNames(self.forumId)
        return Article(self, forumId=self.forumId, **dict(zip(fieldNames, args)))

    def changeType(self, type=0):
        cursor = dblib.getCursor()
        cursor.execute('UPDATE %sThread SET type=%d WHERE threadId=%s' % (self.forumId, int(type), self.threadId))
        self.type = type

    def toSQL(self):
        self.escapedSubjectForDb = escapeTextForDb(self.subject)
        return "(%(threadId)s, %(type)s, '%(mdate)s', "\
            "%(readCount)s, '%(starter)s', '%(lastPoster)s', %(replies)s, "\
            "'%(escapedSubjectForDb)s')" % self.__dict__
    def __repr__(self):
        L = [repr(article) for article in self]
        return '\n'.join(L)

    def toLink(self):
        return '<a href="%s?action=showThread&forumId=%s&threadId=%s">%s</a>' % (
            'pybbs.py', self.forumId, self.threadId, self.subject)


# Article
################################################################################
class Article(Message, Tree):
    def __init__(self, parent, id='NULL', threadId=-1, forumId=-1, offset=0, ip='',
                 date='', mdate='',
                 readCount=0, uid='', upload1='', downCount1=0,
                 htmltag=0, emoticon=0, bbcode=1, notify=0, subject='', body=''):
        self.setParent(parent)
        self.id = self.articleId = id
        self.threadId = threadId
        self.forumId = forumId
        self.offset = offset
        self.ip = ip and ip or os.environ.get('REMOTE_ADDR', 'localhost')
        if date:
            self.date = date
        else:
            self.date = time.strftime(config.databaseDateFormat)
        if mdate:
            self.mdate = mdate
        else:
            self.mdate = self.date
        self.readCount = readCount
        self.uid = uid
        self.upload1 = upload1
        self.downCount1 = int(downCount1)
        self.htmltag = int(htmltag)
        self.emoticon = int(emoticon)
        self.bbcode = bbcode
        self.notify = notify
        self.subject = subject
        self.body = body.rstrip()

    def escapeSubjectAndBody(self):
        self.escapedSubjectForDb = escapeTextForDb(self.subject.rstrip())
        self.escapedBodyForDb = escapeTextForDb(self.body.rstrip())

    def escapeBodyHtml(self):
        if self.htmltag and self.bbcode:
            self.escapedHtmlBody = bbcode.bbCodeProcessor(self.body, escapeWhiteSpace=False)
        elif self.bbcode:
            self.escapedHtmlBody = cgi.escape(self.body)
            self.escapedHtmlBody = bbcode.bbCodeProcessor(self.escapedHtmlBody, escapeWhiteSpace=True)
        elif self.htmltag:
            self.escapedHtmlBody = linkProcessor.linkProcessor(self.body)
        else:
            self.escapedHtmlBody = cgi.escape(self.body)
            self.escapedHtmlBody = escapeWhiteSpace(self.escapedHtmlBody)
            self.escapedHtmlBody = linkProcessor.linkProcessor(self.escapedHtmlBody)
        user = users.getUser(self.uid)
        if self.emoticon:
            import emoticons
            tag = "<img src='./%s' border='0' align='absmiddle'>"
            emo = emoticons.Emoticons().emoticons.items()   # FIXME
            emo.sort(lambda lhs, rhs: cmp(len(rhs[0]), len(lhs[0])))
            for key, value in emo:
                self.escapedHtmlBody = self.escapedHtmlBody.replace(key, (tag % value[1]).replace('\\', '/'))
        self.escapedHtmlBody = escapeTagMarker(self.escapedHtmlBody)

    def toHTML(self, color):
        self.escapeBodyHtml()
        globalNS = getGlobalNS()
        globalNS['articleId'] = self.id
        d = self.makeOneNameSpace()
        d['articleBgColor'] = color
        d['self'] = self
        d['writer'] = users.getUser(self.uid) or users.getUser('anonymous')
        d['uploadFileName'] = os.path.split(self.upload1)[1]
        d['uploadFileIsImage'] = os.path.splitext(self.upload1)[1].lower() in ('.jpg', '.jpeg', '.gif', '.png', '.bmp')
        try:
            forum = self.getParent().getParent()
        except AttributeError:
            forum = Forum(None, 'tmp forum')    # for test code only
        teulPath = os.path.join(config.skinPath, forum.articleSkinPath)
        if self.offset != 0:
            teulPath = os.path.join(config.skinPath, forum.articleSkinPath2)
        template = getTemplate(teulPath)
        return templateProcessor(template, globalNS, d)

    def incrDownCount(self, forumId):
        self.downCount1 += 1
        cursor = dblib.getCursor()
        cursor.execute("UPDATE %s SET downCount1=%s WHERE id=%s" %
                 (forumId, self.downCount1, self.id))

    def toSQL(self, forumId):
        self.escapeSubjectAndBody()
        fieldInfo = dblib.getFieldInfo(forumId)
        fieldInfo = [f[:2] for f in fieldInfo]
        L = []
        for fieldName, fieldType in fieldInfo:
            if fieldName == 'subject': fieldName = 'escapedSubjectForDb'
            if fieldName == 'body': fieldName = 'escapedBodyForDb'
            fieldType = fieldType.lower()
            if fieldType.find('char') >= 0 or fieldType.find('datetime') >= 0 or fieldType.find('text') >= 0 :
                L.append("'%%(%s)s'" % fieldName)
            else:
                L.append('%%(%s)s' % fieldName)
        return '(%s)' % (', '.join(L) % self.__dict__)
    
        fieldNames[fieldNames.lower().find('subject')] = 'escapedSubjectForDb'
        fieldNames[fieldNames.lower().find('body')] = 'escapedBodyForDb'
        
        return "(%(id)s, %(threadId)s, %(offset)s, '%(ip)s', "\
            "'%(date)s', '%(mdate)s', "\
            "%(readCount)s, '%(uid)s', '%(upload1)s', '%(downCount1)s', "\
            "%(htmltag)s, %(emoticon)s, %(bbcode)s, %(notify)s, '%(escapedSubjectForDb)s', '%(escapedBodyForDb)s')" % self.__dict__
    def __repr__(self):
        return """id : %(id)s
threadId : %(threadId)s
offset : %(offset)s
ip : '%(ip)s'
date : '%(date)s'
mdate : '%(mdate)s'
readCount : %(readCount)s
uid : '%(uid)s'
upload1 : '%(upload1)s'
htmltag : '%(htmltag)s'
emoticon : '%(emoticon)s'
bbcode : '%(bbcode)s'
notify : '%(notify)s'
subject : '%(subject)s'
body : '%(body)s'
""" % self.__dict__

    def toLink(self):
        return '<a href="%s?action=showThread&forumId=%s&threadId=%s#%s">%s</a>' % (
            'pybbs.py', self.forumId, self.threadId, self.articleId, self.subject)

    ########### for thumb nail #####################
    def getImageFileNameList(self, cols):
        imageFileNameList = []
        dirPath = os.path.split(self.upload1)[0]
        for fullPath in glob.glob(dirPath+'/*'):
            fullPath = fullPath.replace('\\', '/')
            fname = os.path.split(fullPath)[1]
            ext = os.path.splitext(fname)[1]
            if not fname.startswith('thumb_') and ext.lower() in ['.jpg', '.jpeg', '.gif', '.png']:
                imageFileNameList.append(fname)
        imageFileNameList.sort(lambda a, b: cmp(a.lower(), b.lower()))

        rowList = []
        k = 0
        for k in range(cols, len(imageFileNameList), cols):
            rowList.append(imageFileNameList[k-cols:k])
        else:
            rowList.append(imageFileNameList[k:] + [''] * (cols-len(imageFileNameList)+k))

        return imageFileNameList, rowList

    def getThumbnailFileNameList(self, cols=5):
        thumbnailList = []
        dirPath = os.path.split(self.upload1)[0]
        for fullPath in glob.glob(dirPath+'/*'):
            fullPath = fullPath.replace('\\', '/')
            fname = os.path.split(fullPath)[1]
            ext = os.path.splitext(fname)[1]
            if fname.startswith('thumb_') and ext.lower() in ['.jpg', '.jpeg', '.gif', '.png']:
                thumbnailList.append(fname)
        thumbnailList.sort(lambda a, b: cmp(a.lower(), b.lower()))

        rowList = []
        k = 0
        for k in range(cols, len(thumbnailList), cols):
            rowList.append(thumbnailList[k-cols:k])
        else:
            rowList.append(thumbnailList[k:] + [''] * (cols-len(thumbnailList)+k))
        return thumbnailList, rowList

    def unzipImageFilesAndMakeThumbnails(self, sizeX=128, sizeY=96):
        try:
            import Image 
        except ImportError:
            return
        z = zipfile.ZipFile(self.upload1)
        errList = z.testzip()
        if errList:
            raise ZipCRCError, '%s' % fileName
        dirPath = os.path.split(self.upload1)[0]
        for fname in z.namelist():
            s = z.read(fname)
            ext = os.path.splitext(fname)[1]
            if ext.lower() not in ['.jpg', '.jpeg', '.gif', '.png']:
                continue
            fname = os.path.split(fname)[1]
            newFname = dirPath+'/'+ fname
            thumbFname = dirPath+'/thumb_'+fname
            open(newFname, 'wb').write(s)
            im = Image.open(newFname)
            im.thumbnail((sizeX, sizeY), Image.ANTIALIAS)
            im.save(thumbFname)

    def removeUploadedFiles(self, fileNameList):
        dirPath = os.path.split(self.upload1)[0]
        for fname in fileNameList:
            fPath = os.path.join(dirPath, fname)
            if os.path.isfile(fPath):
                os.remove(fPath)

def readArticle(forumId, id):
    sql = 'SELECT * FROM %s WHERE id=%s' % (forumId, id)
    article = articleCache.getItem(sql)
    if article: return article
    cursor = dblib.getCursor()
    cursor.execute(sql)
    args = cursor.fetchone()
    fieldNames = dblib.getFieldNames(forumId)
    try:
        article = Article(None, forumId=forumId, **dict(zip(fieldNames, args)))
        articleCache.putItem(sql, article)
        return article
    except TypeError:
        return None

class PageList(list, Page):
    def __init__(self, pagesize, startPage=1):
        self.pagesize = int(pagesize)
        self.startPage = startPage

def fileUpload(forumId, article, fileStr):
    import re
    path, fname = os.path.split(article.upload1)
    dir = '%s-%s' % (forumId, article.threadId)
    dir = os.path.join(config.uploadPath, dir)
    if not os.path.exists(dir):
        os.makedirs(dir)
    filePath = os.path.join(dir, fname).replace(' ', '_')
    if os.path.exists(filePath):
        p = re.compile('.+\[(\d+)\].+')
        m = p.match(filePath)
        while m:
            n = int(m.group(1))
            start, end = m.span(1)
            filePath = '%s%s%s' % (filePath[:start], n+1, filePath[end:])
            if not os.path.exists(filePath):
                break    
            m = p.match(filePath)
        else:
            filePath = '%s[1]%s' % os.path.splitext(filePath)
    open(filePath, 'wb').write(fileStr)
    article.upload1 = filePath.replace('\\', '/')
    return article.upload1

def writeThread(forumId, article, fileName='', fileStr='', type=DEFAULT, readCount=0, forumType='Discussion'):
    if article.htmltag:
        htmlValidate.validate(article.body)
    if article.bbcode:
        bbcode.bbCodeProcessor(article.body)
    cursor = dblib.getCursor()
    sqlarg = "(NULL, %s, '%s', %s, '%s', '%s', 0, '%s')" % (
        type, article.mdate, readCount, article.uid, article.uid, escapeTextForDb(article.subject))
    cursor.execute("INSERT INTO %sThread VALUES %s" % (forumId, sqlarg))
    newThreadId = cursor.insert_id()
    article.threadId = newThreadId

    if fileName:
        fileName = fileName.replace('\\', '/')
        article.upload1 = os.path.split(fileName)[1]
        fileUpload(forumId, article, fileStr)
    
        
    names = dblib.getFieldNames(forumId)
    cursor.execute("INSERT INTO %s (%s) VALUES %s" % (forumId, ', '.join(names), article.toSQL(forumId)))
    article.articleId = cursor.insert_id()

    user = users.getUser(article.uid)
    if user:
        user.incrPostCount()
    refreshForum(forumId)

    if type == DEFAULT and forumType == 'PhotoGallery' and article.upload1:
        try:
            groups = getGroups()
            forum = groups.getForumById(forumId)
            article.unzipImageFilesAndMakeThumbnails(forum.thumbnailSizeX, forum.thumbnailSizeY)
        except zipfile.BadZipfile, ZipCRCError:
            deleteThread(forumId, article.threadId)
            raise BadGalleryFile
    return article.threadId

def getMaxArticleOffset(cursor, forumId, threadId):
    cursor.execute('SELECT MAX(offset) from %s WHERE threadId=%s' % (forumId, threadId))
    max = cursor.fetchone()[0]
    if not max:
        return 0
    return max

def replyArticle(forumId, article, threadId, fileName='', fileStr=''):
    if article.htmltag:
        htmlValidate.validate(article.body)
    if article.bbcode:
        bbcode.bbCodeProcessor(article.body)
    cursor = dblib.getCursor()
    article.threadId = threadId
    article.offset = getMaxArticleOffset(cursor, forumId, threadId) + 1   # Critical Section?
    if article.date[:4] == '0000':
        article.setCurrentDateAndTime()
    #article.setModifiedDateAndTime()
    if fileName:
        fileName = fileName.replace('\\', '/')
        article.upload1 = os.path.split(fileName)[1]
        fileUpload(forumId, article, fileStr)
    names = dblib.getFieldNames(forumId)
    cursor.execute("INSERT INTO %s (%s) VALUES %s" % (forumId, ', '.join(names), article.toSQL(forumId)))
    newArticleId = cursor.insert_id()
    cursor.execute("UPDATE %sThread SET replies=replies+1, lastPoster='%s', mdate='%s' WHERE threadId=%s" %
                 (forumId, article.uid,
                  article.mdate, threadId))
    thread = getGroups().getThread(forumId, threadId)
    thread.replies += 1
    thread.mdate = article.mdate
    thread.lastPoster = article.uid
    user = users.getUser(article.uid)
    if user:
        user.incrPostCount()
    refreshForum(forumId)
    return article.offset, newArticleId

def editArticle(forumId, articleId, article, uid, fileName='', fileStr='', remove=False):
    '''update subject and body text'''
    if article.htmltag:
        htmlValidate.validate(article.body)
    if article.bbcode:
        bbcode.bbCodeProcessor(article.body)
    cursor = dblib.getCursor()
    article.setModifiedDateAndTime()
    if remove or fileName:
        if article.upload1:
            os.remove(article.upload1)
        article.upload1 = ''
        article.downCount1 = 0
    if fileName:
        fileName = fileName.replace('\\', '/')
        article.upload1 = os.path.split(fileName)[1]
        fileUpload(forumId, article, fileStr)
        article.downCount1 = 0
    article.escapeSubjectAndBody()
    cursor.execute("UPDATE %s SET mdate='%s', upload1='%s', downCount1=%s, htmltag=%s, emoticon=%s, bbcode=%s, notify=%s,subject='%s', body='%s' WHERE id=%s" %
                 (forumId, article.mdate, article.upload1, article.downCount1,
                  article.htmltag, article.emoticon, article.bbcode, article.notify,
                  article.escapedSubjectForDb, article.escapedBodyForDb, articleId))
    cursor.execute("UPDATE %sThread SET mdate='%s', lastPoster='%s' WHERE threadId=%s" %
                 (forumId, article.mdate, uid, article.threadId))
    if article.offset == 0:
        cursor.execute("UPDATE %sThread SET subject='%s' WHERE threadId=%s" %
                 (forumId, article.escapedSubjectForDb, article.threadId))
    key = 'SELECT * FROM %s WHERE id=%s' % (forumId, articleId)
    thread = getGroups().getThread(forumId, article.threadId)
    thread.mdate = article.mdate
    thread.subject = article.subject
    thread.escapedHtmlSubject = escapeWhiteSpace(cgi.escape(thread.subject))
    thread.lastPoster = uid
    refreshForum(forumId)
    articleCache.deleteItem(key)

def deleteThread(forumId, threadId):
    cursor = dblib.getCursor()
    cursor.execute('SELECT id FROM %s WHERE threadId=%s' % (forumId, threadId))
    idList = []
    for articleId, in cursor.fetchall():
        deleteArticle(forumId, articleId, '')
        idList.append(articleId)
    cursor.execute('DELETE FROM %sThread WHERE threadId=%s' % (forumId, threadId))
    uploadPath = os.path.join(config.uploadPath, '%s-%s' % (forumId, threadId))
    if os.path.exists(uploadPath):
        shutil.rmtree(uploadPath)
    refreshForum(forumId)
    return idList

def deleteArticle(forumId, articleId, uid):
    cursor = dblib.getCursor()
    cursor.execute('SELECT threadId, offset, upload1 FROM %s WHERE id=%s' % (forumId, articleId))
    threadId, offset, upload1 = cursor.fetchall()[0]
    article = readArticle(forumId, articleId)
    writer = users.getUser(article.uid)
    if writer:
        writer.decrPostCount()
    cursor.execute('DELETE FROM %s WHERE threadId=%s AND offset=%s' % (forumId, threadId, offset))
    cursor.execute("UPDATE %sThread SET replies=replies-1, mdate='%s', lastPoster='%s' WHERE threadId=%s" % (forumId, time.strftime(config.databaseDateFormat), uid, threadId))
    if upload1:
        try:
            os.remove(upload1)
        except OSError:
            pass
    thread = getGroups().getThread(forumId, article.threadId)
    thread.replies -= 1
    thread.setModifiedDateAndTime()
    thread.lastPoster = uid
    refreshForum(forumId)
    return True

def clearTable(forumId):
    cursor = dblib.getCursor()
    cursor.execute('DELETE FROM %s' % self.forumId)

def refreshForum(forumId):
    groups = getGroups()
    forum = groups.getForumById(forumId)
    forum.refresh()

def mergeForumById(source, target):
    groups = getGroups()
    srcForum = groups.getForumById(source)
    tarForum = groups.getForumById(target)
    srcForum.refresh()
    for thread in srcForum:
        tarForum.copyThread(thread)
    for thread in srcForum.getNoticeList():
        tarForum.copyThread(thread)
    srcForum.getParent().deleteForumById(srcForum.forumId)

def getInvalidForumNames(forumNames):
    groups = getGroups()
    existingForumNameList = map(lambda forum: forum[1], groups.getForumList())
    notExistForumNames = []
    for forumName in forumNames:
        if not forumName in existingForumNameList:
            notExistForumNames.append(forumName)
    return notExistForumNames


def getInvalidUserGroups(registeredUserGroupList, userGroupList):
    L = []
    for userGroupName in userGroupList:
        if userGroupName not in registeredUserGroupList:
            L.append(userGroupName)
    return L

#################################################################################
def getForumGroupEmailList(forumId):
    groups = getGroups()
    forum = groups.getForumById(forumId)
    userGroupObj = userGroup.getUserGroup()
    userGroupEmailList = userGroupObj.getEmailListOfGroups(forum.emailNoticeUserGroupList)
    return userGroupEmailList

def sendArticleChangeNotificationEmail(user, forumId, actionName, url, subject='', body=''):
    fname = os.path.join(config.skinPath, 'emailNotify_%s.teul' % actionName)
    try:
        uidList = open(os.path.join(config.mailNoticeFolder, 'MailNotice_%s') % forumId).read().strip().split('\n')
    except IOError:
        uidList = []
    groups = getGroups()
    forum = groups.getForumById(forumId)
    emailList = []
    for uid in uidList:
        user2 = users.getUser(uid)
        if user2 and checkForumAdmission(forum, forum.view, user2):
            emailList.append(user2.email)
    userGroupEmailList = getForumGroupEmailList(forumId)
    for email in userGroupEmailList:
        if email not in emailList:
            emailList.append(email)

    if user.email in emailList:
        emailList.remove(user.email)

    forumName = forum.forumName
    body = templateProcessor(open(fname).read(), locals(), globals())
    try:
        m = mail.Mail()
        m.send('"%s"<%s>' % (config.siteName, config.webmaster), emailList, '[%s] article changed (%s)' % (config.siteName, actionName), body, _subtype='html')
    except mail.SendMailFailed:
        config.errlogger.log(__name__, 'mailNotification_failure SendMailFailed')
    except mail.SMTPError:
        config.errlogger.log(__name__, 'mailNotification_failure SMTPError')

def initializeForumEmailNotificationFiles():
    import glob
    if not os.path.exists(config.mailNoticeFolder):
        os.makedirs(config.mailNoticeFolder)
    for fname in glob.glob(os.path.join(config.mailNoticeFolder, 'MailNotice_*')):
        os.remove(fname)
    groups = getGroups()
    for user in users.getUserList(pagesize=users.numberOfUsers(), updateCache=False):
        for forumName in user.getForumEmailNoticeList():
            forum = groups.getForumByName(forumName)
            forum.addUserIdToForumEmailNoticeListFile(user.uid)

def getForumTypeByForumId(forumId):
    groups = getGroups()
    forum = groups.getForumById(forumId)
    return forum.defaultType
 
def hasBannedExtension(fileName):
    bannedExtList = map(str.lower, config.bannedExtensions.split())
    for bannedExt in bannedExtList:
        if fileName.lower().find('.'+bannedExt) >= 0:
            return True
    return False

# following functions are used by skins
################################################################################
def getCookedDateFormat(date, format=None):
    if not date:
        return None
    if format == None:
        format = config.dateFormat
    import time
    try:
        ymd, hms= date.split()
        date = map(lambda field: int(field), ymd.split('-') + hms.split(':')) + [0] * 3
        return time.strftime(format, date)
    except TypeError:
        return ''
    except:
        return date


def strCutter(text, length=40, postfix="..."):
    if len(text) < length: return text
    pos = length - 1 
    char = text[pos] 
    while ord(char) > 127 and pos >= 0:
        pos -= 1
        char = text[pos]
    return text[:(length-pos) % 2 and length or length - 1] + postfix
