'''
    PyBB - Config Module

    Copyright (c) 2003-2004 Gang Seong Lee <gslee@mail.kw.ac.kr>
    All rights reserved, see COPYING for details.
'''

from __future__ import generators
import cgi
import os
import glob
import types
import time
import string

import pyConfigParser
from templateProcessor import templateProcessor, getTemplate
import logger

setupFileName = 'setup.cgi'

class Configuration:
    formType = {'dbname':'hidden', 'dbhost':'hidden', 'dbuser':'hidden', 
                'dbpassword':'hidden', 'tablePrefix':'hidden',
                'groupDescFileName':'hidden', 'sessionDir':'hidden',
                'systemFolder':'hidden',
                }

    intList = ['HTTP_PORT', 'numberOfSearchResultsInAPage', 'numberOfUsersInAPage'
        'anonymousLevel', 'waitingLevel', 'memberLevel', 'adminLevel',
        'sessionRemoveProbability', 'sessionLife', 'maxFileSize', 
        'maxUserCache', 'maxUserListCache', 'maxArticleCache', 'maxThreadCachePerForum',
        'maxUserListPages',
        'allowedMaxNumberOfArticlesToWritePerIpADay', 'allowedMaxNumberOfArticlesToReadPerIpADay',
        ]
    floatList = [
        'readBlockingTime', 'writeBlockingTime',
        ]
    boolList = [
        'newUserEmailNotification',
        'userMembershipApprovalRequired', 'errorLogging', 
        'machineBlocking',
        ]

    def __init__(self):
        self.dict = {}
        self.load()
        
    def load(self):
        cfg = pyConfigParser.CookedConfigParser()
        cfg.readfp(open(setupFileName))
        for section in cfg.sections():
            for option in cfg.options(section):
                if option in self.intList:
                    value = cfg.getint(section, option)
                elif option in self.floatList:
                    value = cfg.getfloat(section, option)
                elif option in self.boolList:
                    value = cfg.getboolean(section, option)
                else:
                    value = cfg.get(section, option)
                if section.lower() == 'paths':
                    while value and value[0] == '/':
                        value = value[1:]
                    value = os.path.normpath(value)
                    value = value.replace('\\', '/')
                self.dict[option] = value
        self.sections = cfg.sections()
    def __getattr__(self, name):
        return self.dict[name]

    def makeItGlobal(self):
        d = globals()
        d.update(self.dict)
        for key, val in cfg_defaults.items():
            if key.endswith('TableName'):
                val = self.dict.get('tablePrefix', cfg_defaults['tablePrefix']) +'_' + val
                d[key] = val
            if key in ['mailNoticeFolder', 'ipControlFolder']:
                d[key] = os.path.join(systemFolder, val)
        d['skinImagePath'] = skinPath + '/images'

    def updateOne(self, section, option, value):
        cfg = pyConfigParser.CookedConfigParser()
        cfg.readfp(open(setupFileName))
        if not (cfg.has_section(section) and cfg.has_option(section, option)):
            return False
        cfg.set(section, option, value)
        cfg.write(open(setupFileName, 'w'))
        self.load()
        return True
    
    def toHTML(self, globalNS):
        html = 'pybbsConfig.teul'
        template = getTemplate(os.path.join(skinPath, html))
        sections = self.sections.copy()
        theseSectionsDoNotPrint = ['DB setup', 'paths']
        sections = pyConfigParser.MyDict()
        for section in self.sections:
            sections[section] = pyConfigParser.MyDict()
            for option in self.sections[section]:
                if option != '__name__':
                    sections[section][option] = self.sections[section][option]
        #sections['site information']['skinPath'] = self.sections['paths']['skinPath']
        #for section in theseSectionsDoNotPrint:
        #    del sections[section]
        localNS = locals()
        localNS['formType'] = self.formType
        return templateProcessor(template, globalNS, locals())

def loadCategory(conf):
    iniList = open(os.path.join(systemFolder, conf.groupDescFileName)).readlines()
    iniList = filter(None, iniList)
    global dbTables
    dbTables = []
    pos = 0
    for fname in iniList:
        fname = fname.strip()
        cfg = pyConfigParser.CookedConfigParser()
        cfg.readfp(open(os.path.join(systemFolder, fname)))
        cat = CategoryConfig( [decode2Str(os.path.splitext(fname)[0])], cfg.get('CategoryConfiguration', 'Fold') )
        dbTables.append(cat)
        cfg.remove_section('CategoryConfiguration')
        for section in cfg.sections():
            forumCfg = {}
            forumCfg['forumId'] = section
            for option in cfg.options(section):
                field = option[0].lower() + option[1:]
                forumCfg[field] = cfg.get(section, option)
            dbTables[pos].append(ForumConfig(forumCfg))
        pos += 1

def saveGroupDescFile():
    newIniList = []
    for category in dbTables:
        categoryName = category.getName()
        forums = [x for x in category if type(x) == ForumConfig]
        cfg = pyConfigParser.CookedConfigParser()
        cfg.add_section('CategoryConfiguration')
        cfg.set('CategoryConfiguration', 'Fold', category.fold)
        for forum in forums:
            forumId = forum.forumId
            cfg.add_section(forumId)
            for key, value in forum.items():
                if key == 'sortBy':
                    key = 'sort'
                cfg.set(forumId, key[0].upper()+key[1:], value)
        cfg.write(open(os.path.join(systemFolder, encode2Hex(categoryName)+'.ini'), 'w'))
        newIniList.append(encode2Hex(categoryName)+'.ini')
    open(os.path.join(systemFolder, groupDescFileName), 'w').write('\n'.join(newIniList))


class CategoryConfig(list):
    attrList = 'fold'.split()
    def __init__(self, L, fold='folding'):
        list.__init__(self, L)
        self.fold = fold
    def getName(self):
        return self[0]
    def __getattr__(self, name):
        return self[self.attrList.index(name)]
    def getForumConfigs(self):
        for forum in self[1:]:
            yield forum
    __iter__ = getForumConfigs

class ForumConfig(dict):
    def __getattr__(self, name):
        return self[name]
    def __setattr__(self, name, value):
        self[name] = value

def getCategories():
    for category in dbTables:
        yield category

def getForumNameById(forumId):
    for category in dbTables:
        for forum in category:
            if forum.forumId == forumId:
                return forum['forumName']
    return False

def getForumConfigById(forumId):
    for category in dbTables:
        for forum in category:
            if forum.forumId == forumId:
                return forum
    return False

def getHomeUrl():
    global urlPrefix
    if urlPrefix.startswith('/'):
        urlPrefix = urlPrefix[1:]
    if urlPrefix.endswith('/'):
        urlPrefix = urlPrefix[:-1]
    path = urlPrefix and urlPrefix+'/' or ''
    if HTTP_PORT == 80:
        return 'http://%s/%s%s' % (HOST_URL, path, 'pybbs.py')
    return 'http://%s:%s/%s%s' % (HOST_URL, HTTP_PORT, path, 'pybbs.py')

cfg_defaults = {
    'databaseEngine':'mysql',
    'dbTables' : [],
    'tablePrefix' : 'pybb',
    'userTableName': 'user',
    'memoTableName' : 'memo',
    'emoticonTableName': 'emoticons',
    'dateForamt': '%Y-%m-%d %H:%M:%S', 
    'databaseDateFormat': '%Y-%m-%d %H:%M:%S',
    'subjectLength' : 90,

    'userGroupConfigFile':'__userGroup__.ini',
    'groupDescFileName': 'groupDesc.txt',
    'maxUserCache': 30,
    'UserIdLengthMin':3,
    'UserIdLengthMax':32,

    'siteName':'PyBB',
    'HTTP_PORT':9080,
    'urlPrefix':'',
    'smtpHOST' : 'localhost',
    'smtpCharset' : 'euc-kr',

    'sessionLife' : 1000,
    'sessionRemoveProbability' : 100,
    'sessionDir' : './Session/',

    'adminLevel' : 10,
    'memberLevel' : 2,
    'waitingLevel' : 1,
    'anonymousLevel' : 0,
    
    'avatarFilePath': 'images/avatars',
    'avatarExts': ['.jpg', '.jpeg', '.gif', '.png'],
    'skinPath': 'skins/default',
    'emoticonPath': 'emoticons',
    'uploadPath': 'upload',
    'maxFileSize':1048576 * 10,
    'systemFolder':'./systemFolder', 

    'numberOfSearchResultsInAPage': 10,
    'numberOfArticlesInAPage': 10,
    'numberOfTitlesInAPage': 10,

    'maxUserCache' : 20,
    'maxUserListCache' : 5,
    'maxArticleCache' : 100,
    'maxThreadCachePerForum' : 200,

    'newUserEmailNotification':True,

    'machineBlocking':False,
    'readBlockingTime':0.5,
    'writeBlockingTime':3,

    'allowedMaxNumberOfArticlesToWritePerIpADay':30,
    'allowedMaxNumberOfArticlesToReadPerIpADay':100,

    'allowIpList':'',
    'denyIpList':'',

    'mailNoticeFolder':'mailNotice',
    'ipControlFolder':'ipControl',

    'bannedExtensions': 'php htm',
}

def makeDefaultConfigGlobal():
#    for key, val in cfg_defaults.items():
#        if key not in globals():
#        if key.endswith('TableName'):
#        val = globals().get('tablePrefix', cfg_defaults['tablePrefix']) +'_' + val
#        globals()[key] = val
    globals().update(cfg_defaults)
    globals()['skinImagePath'] = skinPath + '/images'

######################        
def reload():
    conf.load()
    conf.makeItGlobal()
    loadCategory(conf)

######################        
allowedChars = string.ascii_letters + string.digits
def encode2Hex(s):
    L = []
    for ch in s:
        if ch not in allowedChars:
            ch = '_%x' % ord(ch)
        L.append(ch)
    return ''.join(L)

def decode2Str(s):
    chList = s.split('_')
    if len(chList) < 2:
        return s
    res = ''
    isStart = True
    for ch in chList:
        if len(ch) < 2:
            res += ch
        else:
            if isStart and chList:
                res += ch
            else:
                res += chr(int(ch[:2], 16))
                res += ch[2:]
        isStart = False
    return res

makeDefaultConfigGlobal()
conf = Configuration()
errlogger = logger.ErrorLogger(conf.errorLogging)
reload()
