'''
    PyBB - Mail module

    Copyright (c) 2003-2004 Gang Seong Lee <gslee@mail.kw.ac.kr>
    All rights reserved, see COPYING for details.
'''
import os
import smtplib
from email.MIMEText import MIMEText
from templateProcessor import templateProcessor, getTemplate
import config

class SendMailError(Exception):
    pass
class SendMailFailed(SendMailError):
    pass

class SMTPError(SendMailError):
    pass

class Mail:
    def __init__(self):
        try:
            self.smtp = smtplib.SMTP(config.smtpHOST)   # SMTPConnectError if failed
        except:
            raise SMTPError, 'smtp open error'

    def __del__(self):
        if hasattr(self, 'smtp'):
            self.smtp.quit()
            
    def send(self, _from, _to, subject, contents, _subtype='plain'):
        if not hasattr(self, 'smtp'):
            return
        msg = MIMEText(contents, _subtype=_subtype, _charset=config.smtpCharset)
        msg['Subject'] = subject
        msg['From'] = _from
        try:
            if type(_to) == type([]):
                msg['To'] = ', '.join(_to)
                self.smtp.sendmail(_from, _to, msg.as_string())
            else:
                msg['To'] = _to
                self.smtp.sendmail(_from, [_to], msg.as_string())
        except:
            raise SendMailFailed, 'from %s to %s' % (_from, _to)

def sendEmailToUsers(userList, teulPath, subject='', subtype='html', localNS={}, globalNS={}):
    fname = os.path.join(config.skinPath, teulPath)
    subject = '[%s]%s' % (config.siteName, subject)
    try:
        m = Mail()
        for user in userList:
            body = templateProcessor(open(fname).read(), localNS, globalNS)
            m.send('"%s"<%s>' % (config.siteName, config.webmaster), [user.email], subject, body, _subtype=subtype)
    except SendMailFailed:
        config.errlogger.log(__name__, 'mailNotification_failure SendMailFailed')
    except SMTPError:
        config.errlogger.log(__name__, 'mailNotification_failure SMTPError')
